import threading
import logging
import gobject
import gst
import sys

from spotify.audiosink import BaseAudioSink

logger = logging.getLogger('spotify.audiosink.gstreamer')
gobject.threads_init()

CAPS_TEMPLATE = """
   audio/x-raw-int,
   endianness=(int)%(endianness)s,
   channels=(int)%(channels)d,
   width=(int)16,
   depth=(int)16,
   signed=(boolean)true,
   rate=(int)%(sample_rate)d
"""

class GstreamerSink(BaseAudioSink):
    """
    Audio sink wrapper for systems with Gstreamer installed, which may include
    Linux, Mac OS X, and Windows systems.
    """

    def __init__(self, **kwargs):
        super(GstreamerSink, self).__init__(**kwargs)
        if sys.byteorder == 'little':
            self._endianness = '1234'
        elif sys.byteorder == 'big':
            self._endianness = '4321'
        caps_string = CAPS_TEMPLATE % {
            'endianness': self._endianness,
            'sample_rate': 44100,
            'channels': 2,
        }
        caps = gst.caps_from_string(caps_string)
        self._pipeline = gst.parse_launch(' ! '.join([
            'appsrc name="application_src" block=true',
            'audioconvert',
            'autoaudiosink',
        ]))
        self._source = self._pipeline.get_by_name('application_src')
        self._source.set_property('caps', caps)
        self.mainloop = None
        self.mainloop_thread = threading.Thread(target=self.start_glib)
        self.mainloop_thread.setDaemon(True)
        self.mainloop_thread.start()
        self._setup_message_processor()

    def start_glib(self):
        self.mainloop = gobject.MainLoop()
        self.mainloop.run()
        logger.debug('Mainloop running')

    def _setup_message_processor(self):
        bus = self._pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self._on_message)

    def _on_message(self, bus, message):
        if message.type == gst.MESSAGE_EOS:
            logger.debug('Track ended')
            self._pipeline.set_state(gst.STATE_NULL)
            self.backend.next()

    def end_of_track(self):
        self._source.emit('end-of-stream')

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        assert sample_type == 0, u'Expects 16-bit signed integer samples'
        caps_string = CAPS_TEMPLATE % {
            'endianness': self._endianness,
            'sample_rate': sample_rate,
            'channels': channels,
        }
        caps = gst.caps_from_string(caps_string)
        buffer_ = gst.Buffer(frames)
        buffer_.set_caps(caps)
        self._source.set_property('caps', caps)
        self._source.emit('push-buffer', buffer_)
        return num_frames

    def start(self):
        self._pipeline.set_state(gst.STATE_READY)
        self._pipeline.set_state(gst.STATE_PLAYING)

    def stop(self):
        self._pipeline.set_state(gst.STATE_NULL)

    def pause(self):
        self._pipeline.set_state(gst.STATE_PAUSED)



# encoding: utf-8

import unittest
from spotify._mockspotify import mock_track, mock_album, mock_artist, mock_session

class TestTrack(unittest.TestCase):

    artists = [mock_artist('artist1'), mock_artist('artist2')]
    album = mock_album('album1', artists[0])
    track = mock_track(u'æâ€êþÿ', artists, album, 10, 20, 30, 40)

    def test_artists(self):
        self.assertEqual([x.name() for x in self.track.artists()], ['artist1',
                                                                    'artist2'])

    def test_album(self):
        self.assertEqual(self.track.album().name(), "album1")

    def test_name(self):
        self.assertEqual(self.track.name(), u'æâ€êþÿ')

    def test_duration(self):
        self.assertEqual(self.track.duration(), 10)

    def test_popularity(self):
        self.assertEqual(self.track.popularity(), 20)

    def test_disc(self):
        self.assertEqual(self.track.disc(), 30)

    def test_index(self):
        self.assertEqual(self.track.index(), 40)

    def test_error(self):
        self.assertEqual(self.track.error(), 0)

    def test_is_loaded(self):
        self.assertEqual(self.track.is_loaded(), 1)

    def test_starred(self):
        session = mock_session()
        self.assertEqual(self.track.starred(session), False)
        self.track.starred(session, set=True)
        self.assertEqual(self.track.starred(session), True)
        self.track.starred(session, set=False)
        self.assertEqual(self.track.starred(session), False)

    def test_availability(self):
        self.assertEqual(self.track.availability(), 1)

    def test_is_local(self):
        self.assertFalse(self.track.is_local())



class SpotifyContainerManager:
    """
    Handles Spotify playlist container callbacks. To implement you own
    callbacks, inherit from this class.

    Exceptions raised in your callback handlers will be displayed on the
    standard error output (stderr).
    """

    def __init__(self):
        pass

    def watch(self, container, userdata=None):
        """
        Listen to modifications events on a container.
        """
        container.add_loaded_callback(self.container_loaded, userdata)
        container.add_playlist_added_callback(self.playlist_added, userdata)
        container.add_playlist_moved_callback(self.playlist_moved, userdata)
        container.add_playlist_removed_callback(self.playlist_removed,
                                                userdata)

    def unwatch(self, container, userdata=None):
        """
        Stop listenning to events on the container.
        """
        try:
            container.remove_callback(self.container_loaded, userdata)
            container.remove_callback(self.playlist_added, userdata)
            container.remove_callback(self.playlist_moved, userdata)
            container.remove_callback(self.playlist_removed, userdata)
        except:
            pass

### Callbacks

    def container_loaded(self, container, userdata):
        """
        Callback

        This callback is called when the container has finished loading. This
        event happens after a successful login, or after a modification on the
        container (after the 3 callbacks described below).

        :param container:   a playlist container
        :type container:    :class:`spotify.PlaylistContainer`
        """
        pass

    def playlist_added(self, container, playlist, position, userdata):
        """
        Callback

        Called when a new playlist is added to the container.

        :param container:   a playlist container
        :type container:    :class:`spotify.PlaylistContainer`
        :param playlist:    playlist added to the container
        :type playlist:     :class:`spotify.Playlist`
        :param position:    new position of the playlist
        :type position:     :class:`int`
        """
        pass

    def playlist_moved(self, container, playlist, position, new_position,
            userdata):
        """
        Callback

        Called when a playlist is moved in the container.

        :param container:   a playlist container
        :type container:    :class:`spotify.PlaylistContainer`
        :param playlist:    playlist moved
        :type playlist:     :class:`spotify.Playlist`
        :param position:    old position
        :type position:     :class:`int`
        :param new_position:
        :type new_position: :class:`int`
        """
        pass

    def playlist_removed(self, container, playlist, position, userdata):
        """
        Callback

        Called when a playlist is removed from the container.

        :param container:   a playlist container
        :type container:    :class:`spotify.PlaylistContainer`
        :param playlist:    playlist removed
        :type playlist:     :class:`spotify.Playlist`
        :param position:    old position
        :type position:     :class:`int`
        """
        pass



try: # 2.7
    # pylint: disable = E0611,F0401
    from unittest.case import SkipTest
    # pylint: enable = E0611,F0401
except ImportError:
    try: # Nose
        from nose.plugins.skip import SkipTest
    except ImportError: # Failsafe
        class SkipTest(Exception):
            pass



# encoding: utf-8

import unittest

from spotify._mockspotify import User, mock_user

class TestUser(unittest.TestCase):

    def test_str(self):
        user = mock_user(u'foo',u'bar',1);
        self.assertEqual(str(user), u'foo')

    def test_is_loaded(self):
        user = mock_user('','',1);
        self.assertEqual(user.is_loaded(), True)
        user = mock_user('','',0);
        self.assertEqual(user.is_loaded(), False)

    def test_canonical_name(self):
        user = mock_user(u'foo',u'bar',1);
        self.assertEqual(user.canonical_name(), u'foo')

    def test_display_name(self):
        user = mock_user(u'foo',u'bar',1);
        self.assertEqual(user.display_name(), u'bar')

    def test_unicode(self):
        user = mock_user(u'føø',u'bãr',1);
        self.assertEqual(user.canonical_name(), u'føø')
        self.assertEqual(user.display_name(), u'bãr')



import unittest
from spotify._mockspotify import Link, mock_artist, mock_track, mock_album
from spotify._mockspotify import mock_playlist, mock_search, mock_user
from spotify._mockspotify import registry_add, registry_clean
from spotify import SpotifyError

class TestLink(unittest.TestCase):

    user = mock_user('user')
    artist = mock_artist('artist')
    album = mock_album('album', artist)
    playlist = mock_playlist('playlist', [], user)
    search = mock_search('query', [], [], [])
    track = mock_track('track', [artist], album)

    def setUp(self):
        registry_add('spotify:artist:test_artist', self.artist)
        registry_add('spotify:album:test_album', self.album)
        registry_add('spotify:playlist:test_playlist', self.playlist)
        registry_add('spotify:track:test_track', self.track)

    def tearDown(self):
        registry_clean()

    def test_from_string(self):
        s = "spotify:artist:test"
        l = Link.from_string(s)
        self.assertEqual(str(l), "spotify:artist:test")

    def test_from_track(self):
        l2 = Link.from_track(self.track, 42 * 1000)
        self.assertEqual(str(l2), "spotify:track:test_track#00:42")

    def test_from_album(self):
        l2 = Link.from_album(self.album)
        self.assertEqual(str(l2), "spotify:album:test_album")

    def test_from_artist(self):
        l = Link.from_artist(self.artist)
        self.assertEqual(str(l), "spotify:artist:test_artist")

    def test_from_search(self):
        l2 = Link.from_search(self.search)
        self.assertEqual(str(l2), "spotify:search:query")

    def test_from_playlist(self):
        l = Link.from_playlist(self.playlist)
        self.assertEqual(str(l), "spotify:playlist:test_playlist")

    def test_type(self):
        l = Link.from_track(self.track, 0)
        self.assertEqual(l.type(), Link.LINK_TRACK)

    def test_badlink(self):
        self.assertRaises(SpotifyError, Link.from_string, "BADLINK");

    def test_as_track(self):
        l = Link.from_string("spotify:track:test_track")
        t = l.as_track()
        self.assertEqual(str(t), "track")

    def test_as_album(self):
        l = Link.from_string("spotify:album:test_album")
        t = l.as_album()
        self.assertEqual(str(t), "album")

    def test_as_artist(self):
        l = Link.from_string("spotify:artist:test_artist")
        a = l.as_artist()
        self.assertEqual(str(a), "artist")

    def test_as_string(self):
        s = "spotify:track:str_test"
        l = Link.from_string(s)
        self.assertEqual(str(l), "spotify:track:str_test")



# encoding: utf-8

import unittest
import threading

from spotify import _mockspotify
import spotify.manager.session
# monkeypatch for testing
spotify.manager.session.spotify = spotify._mockspotify

from spotify.manager import SpotifySessionManager
from spotify._mockspotify import mock_track, mock_album


class BaseMockClient(SpotifySessionManager):

    cache_location = "/foo"
    settings_location = "/foo"
    application_key = "appkey_good"
    user_agent = "user_agent_foo"

    def __init__(self):
        SpotifySessionManager.__init__(self, "username_good", "password_good")
        self.awoken = threading.Event() # used to block until awoken

class BaseMockUnicodeClient(SpotifySessionManager):

    cache_location = u'/f××'
    settings_location = u'/f××'
    application_key = "appkey_good"
    user_agent = u'üser_âgent_f©©'

    def __init__(self):
        SpotifySessionManager.__init__(self, "username_good", "password_good")
        self.awoken = threading.Event() # used to block until awoken

class TestSession(unittest.TestCase):

    def test_initialisation(self):
        class MockClient(BaseMockClient):
            def logged_in(self, session, error):
                username = session.username()
                self.found_username = username
                session.logout()
                self.disconnect()

        c = MockClient()
        c.connect()
        self.assertEqual(c.username, c.found_username)

    def test_unicode(self):
        class MockClient(BaseMockUnicodeClient):
            def logged_in(self, session, error):
                username = session.username()
                self.found_username = username
                session.logout()
                self.disconnect()

        c = MockClient()
        c.connect()
        self.assertEqual(c.username, c.found_username)

    def NOtest_load(self):
        class MockClient(BaseMockClient):
            def logged_in(self, session, error):
                track = mock_track("foo", 0, mock_album("bar", mock_artist("baz", 1), 0, "", 0), 0, 0, 0, 0, 0, 1)
                session.load(track)
                session.seek(40000)
                session.play(True)
                session.logout()
                self.disconnect()

            def music_delivery(self, sess, mformat, frames):
                pass

        c = MockClient()
        c.connect()



# encoding: utf-8

import unittest
from spotify._mockspotify import Artist, mock_artist

class TestArtist(unittest.TestCase):

    def test_str(self):
        artist = mock_artist("test_name")
        self.assertEqual(str(artist), "test_name")


    def test_is_loaded(self):
        artist = mock_artist("test_name", is_loaded=1)
        self.assertTrue(artist.is_loaded())
        artist = mock_artist("test_name", is_loaded=0)
        self.assertFalse(artist.is_loaded())

    def test_name(self):
        artist = mock_artist("test_name")
        self.assertEqual(artist.name(), "test_name")

    def test_unicode(self):
        artist = mock_artist(u'æâ€êþÿ')
        self.assertEqual(artist.name(), u'æâ€êþÿ')



import logging
import Queue

import spotify

logger = logging.getLogger('pyspotify.manager.session')

class SpotifySessionManager(object):
    """
    Client for Spotify. Inherit from this class to have your callbacks
    called on the appropriate events.

    Exceptions raised in your callback handlers will be displayed on the
    standard error output (stderr).

    When logging in a user, the application can pass one of:
        - `username` + `password`: standard login using a plaintext password
        - nothing: logs in the last user which credentials have been stored
          using `remember_me`.
        - `username` + `login_blob`: the blob is encrypted data from
          *libspotify*, for when a multi-user application wants to use
          the re-login feature. The blob is obtained from the
          :meth:`credentials_blob_updated` callback after a successful
          login to the Spotify AP.
    """

    api_version = spotify.api_version
    cache_location = 'tmp'
    settings_location = 'tmp'
    application_key = None
    appkey_file = 'spotify_appkey.key'
    user_agent = 'pyspotify-example'

    def __init__(self, username=None, password=None, remember_me=False,
                 login_blob=''):
        self._cmdqueue = Queue.Queue()
        self.username = username
        self.password = password
        self.remember_me = remember_me
        self.login_blob = login_blob
        if self.application_key is None:
            self.application_key = open(self.appkey_file).read()

    def connect(self):
        """
        Connect to the Spotify API using the given username and password.

        This method calls the :func:`spotify.connect` function.

        This method does not return before we disconnect from the Spotify
        service.
        """
        session = spotify.connect(self)
        self.loop(session) # returns on disconnect

    def loop(self, session):
        """
        The main loop.

        Processes events from ``libspotify`` and turns some of them into
        callback calls.
        """
        running = True
        timeout = 0
        while running:
            try:
                message = self._cmdqueue.get(timeout=timeout)
                if message.get('command') == 'music_delivery':
                    num_frames = self.music_delivery_safe(
                        session, *message['args'])
                    message['reply_to'].put(num_frames)
                elif message.get('command') == 'process_events':
                    logger.debug('Got message; processing events')
                    timeout = session.process_events() / 1000.0
                    logger.debug('Will wait %.3fs for next message', timeout)
                elif message.get('command') == 'disconnect':
                    logger.debug('Got message; disconnecting')
                    session.logout()
                    running = False
                else:
                    raise ValueError('Unknown message type')
            except Queue.Empty:
                logger.debug(
                    'No message received before timeout. Processing events')
                timeout = session.process_events() / 1000.0
                logger.debug('Will wait %.3fs for next message', timeout)

    def disconnect(self):
        """
        Terminate the current Spotify session.
        """
        self._cmdqueue.put({'command': 'disconnect'})

    def logged_in(self, session, error):
        """
        Callback.

        Called when the login completes. You almost
        certainly want to do something with
        :meth:`session.playlist_container() <spotify.Session.playlist_container>`
        if the login succeded.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        :param error: an error message, :class:`None` if all went well.
        :type error: string or :class:`None`
        """
        pass

    def logged_out(self, session):
        """
        Callback.

        The user has or has been logged out from Spotify.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        """
        pass

    def metadata_updated(self, session):
        """
        Callback.

        The current user's metadata has been updated.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        """
        pass

    def connection_error(self, session, error):
        """
        Callback.

        A connection error occured in ``libspotify``.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        :param error: an error message. If :class:`None`, the connection is
          back.
        :type error: string or :class:`None`
        """
        pass

    def message_to_user(self, session, message):
        """
        Callback.

        An informative message from ``libspotify``, destinated to the user.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        :param message: a message.
        :type message: string
        """
        pass

    def notify_main_thread(self, session=None):
        """
        Callback.

        When this method is called by ``libspotify``, one should call
        :meth:`session.process_events() <spotify.Session.process_events>`.

        If you use the :class:`SessionManager`'s default loop, the default
        implementation of this method does the job. Though, if you implement
        your own loop for handling Spotify events, you'll need to override this
        method.

        .. warning::
            This method is called from an internal thread in libspotify. You
            should make sure *not* to use the Spotify API from within it, as
            libspotify isn't thread safe.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        """
        self._cmdqueue.put({'command': 'process_events'})

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        """
        Callback.

        Called whenever new music data arrives from Spotify.

        You should override this method *or* :meth:`music_delivery_safe`, not both.

        .. warning::
            This method is called from an internal thread in libspotify. You
            should make sure *not* to use the Spotify API from within it, as
            libspotify isn't thread safe.

        :param session: the current session
        :type session: :class:`spotify.Session`
        :param frames: the audio data
        :type frames: :class:`buffer`
        :param frame_size: bytes per frame
        :type frame_size: :class:`int`
        :param num_frames: number of frames in this delivery
        :type num_frames: :class:`int`
        :param sample_type: currently this is always 0 which means 16-bit
            signed native endian integer samples
        :type sample_type: :class:`int`
        :param sample_rate: audio sample rate, in samples per second
        :type sample_rate: :class:`int`
        :param channels: number of audio channels. Currently 1 or 2
        :type channels: :class:`int`
        :return: number of frames consumed
        :rtype: :class:`int`
        """
        try:
            future = Queue.Queue()
            self._cmdqueue.put({
                'command': 'music_delivery',
                'args': (frames, frame_size, num_frames, sample_type, sample_rate,
                    channels),
                'reply_to': future,
            }, block=False)
            return future.get()
        except Queue.Full:
            return 0

    def music_delivery_safe(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        """
        This method does the same as :meth:`music_delivery`, except that it's
        called from the :class:`SpotifySessionManager` loop. You can safely use
        Spotify APIs from within this method.

        You should override this method *or* :meth:`music_delivery`, not both.
        """
        return 0

    def play_token_lost(self, session):
        """
        Callback.

        The playback stopped because a track was played from another
        application, with the same account.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        """
        pass

    def log_message(self, session, message):
        """
        Callback.

        A log message from ``libspotify``.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        :param message: the message.
        :type message: string
        """
        pass

    def end_of_track(self, session):
        """
        Callback.

        Playback has reached the end of the current track.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        """
        pass

    def credentials_blob_updated(self, session, blob):
        """
        Callback.

        Called when storable credentials have been updated, usually called when
        we have connected to the AP.

        .. warning::
            This method is called from an internal thread in libspotify. You
            should make sure *not* to use the Spotify API from within it, as
            libspotify isn't thread safe.

        :param session: the current session.
        :type session: :class:`spotify.Session`
        :param blob: a string which contains an encrypted token that can be
            stored safely on disk instead of storing plaintext passwords.
        """
        pass



class SpotifyPlaylistManager:
    """
    Handles Spotify playlists callbacks. To implement you own callbacks,
    inherit from this class.

    Exceptions raised in your callback handlers will be displayed on the
    standard error output (stderr).
    """

    def __init__(self):
        pass

    def watch(self, playlist, userdata=None):
        """
        Listen to modifications events on a playlist.
        """
        playlist.add_tracks_added_callback(self.tracks_added, userdata)
        playlist.add_tracks_moved_callback(self.tracks_moved, userdata)
        playlist.add_tracks_removed_callback(self.tracks_removed, userdata)

    def unwatch(self, playlist, userdata=None):
        """
        Stop listening to events on the playlist.
        """
        try:
            playlist.remove_callback(self.tracks_added, userdata)
            playlist.remove_callback(self.tracks_moved, userdata)
            playlist.remove_callback(self.tracks_removed, userdata)
        except:
            pass

### Callbacks

    def tracks_added(self, playlist, tracks, position, userdata):
        """
        Callback

        Called when tracks are inserted in the playlist.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param tracks:      list of added tracks
        :type tracks:       list of :class:`spotify.Track`
        :param position:    position in which the tracks where inserted
        :type position:     :class:`int`
        """
        pass

    def tracks_moved(self, playlist, tracks, new_position, userdata):
        """
        Callback

        Called when tracks are moved in the playlist.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param tracks:      list of moved tracks
        :type tracks:       list of :class:`spotify.Track`
        :param new_position: new position of the tracks
        :type new_position: :class:`int`
        """
        pass

    def tracks_removed(self, playlist, tracks, userdata):
        """
        Callback

        Called when tracks are removed from the playlist.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param tracks:      list of removed tracks
        :type tracks:       list of :class:`spotify.Track`
        """
        pass

    def playlist_renamed(self, playlist, userdata):
        """
        Callback

        Called when a playlist has been renamed.
        :meth:`spotify.Playlist.name()` can be used to find out the new name.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        """
        pass

    def playlist_state_changed(self, playlist, userdata):
        """
        Callback

        Called when state changed for a playlist.

        There are three states that trigger this callback:

        - Collaboration for this playlist has been turned on or off.
        - The playlist started having pending changes, or all pending changes
          have now been committed.
        - The playlist started loading, or finished loading.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        """
        pass

    def playlist_update_in_progress(self, playlist, done, userdata):
        """
        Callback

        Called when a playlist is updating or is done updating.

        This is called before and after a series of changes are applied to the
        playlist. It allows e.g. the user interface to defer updating until the
        entire operation is complete.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param done:        whether the update is finished
        :type done:         :class:`bool`
        """
        pass

    def playlist_metadata_updated(self, playlist, userdata):
        """
        Callback

        Called when metadata for one or more tracks in a playlist has been
        updated.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        """
        pass

    def track_created_changed(self, playlist, position, user, when, userdata):
        """
        Callback

        Called when create time and/or creator for a playlist entry changes.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param position:    track's position in the playlist
        :type position:     :class:`int`
        :param user:        creator of the playlist
        :type user:         :class:`spotify.User`
        :param when:        time in seconds since the UNIX Epoch
        :type when:         :class:`int`
        """
        pass

    def track_message_changed(self, playlist, position, message, userdata):
        """
        Callback

        Called when message attribute for a playlist entry changes.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param position:    track's position in the playlist
        :type position:     :class:`int`
        :param message:     new message
        :type message:      :class:`unicode`
        """
        pass

    def track_seen_changed(self, playlist, position, seen, userdata):
        """
        Callback

        Called when seen attribute for a playlist entry changes.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param position:    track's position in the playlist
        :type position:     :class:`int`
        :param seen:        new seen attribute
        :type seen:         :class:`bool`
        """
        pass

    def description_changed(self, playlist, description, userdata):
        """
        Callback

        Called when playlist description has changed.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param description: new description
        :type description:  :class:`unicode`
        """
        pass

    def subscribers_changed(self, playlist, userdata):
        """
        Callback

        Called when playlist subscribers changes (count or list of names).

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        """
        pass

    def image_changed(self, playlist, image, userdata):
        """
        Callback

        Called when playlist image has changed.

        :param playlist:    playlist on which the event occured
        :type playlist:     :class:`spotify.Playlist`
        :param image:       image id of the new image
        :type image:        :class:`str`
        """
        pass



import pyaudio

from spotify.audiosink import BaseAudioSink

class PortAudioSink(BaseAudioSink):
    """
    Audio sink wrapper for systems with PortAudio installed, which may
    include Linux, Mac OS X, and Windows systems.
    """

    def __init__(self, **kwargs):
        super(PortAudioSink, self).__init__(**kwargs)
        self._device = pyaudio.PyAudio()
        self._stream = None

    def _setup_stream(self, sample_rate, channels):
        if self._stream is not None:
            self._stream.close()
            self._stream = None
        self._stream = self._device.open(rate=sample_rate, channels=channels,
                format=pyaudio.paInt16, output=True)

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        self._call_if_needed(self._setup_stream, sample_rate, channels)
        self._stream.write(frames, num_frames=num_frames)
        return num_frames



import unittest
from spotify._mockspotify import mock_playlist, mock_event_trigger
from spotify._mockspotify import mock_album, mock_artist, mock_user, mock_track
from spotify.manager import SpotifyPlaylistManager
from spotify._mockspotify import User, Playlist

callback_called = None

class MyPlaylistManager(SpotifyPlaylistManager):

    def tracks_added(*args):
        global callback_called
        callback_called = 'tracks_added', args

    def tracks_moved(*args):
        global callback_called
        callback_called = 'tracks_moved', args

    def tracks_removed(*args):
        global callback_called
        callback_called = 'tracks_removed', args

    def playlist_renamed(*args):
        global callback_called
        callback_called = 'playlist_renamed', args

    def playlist_state_changed(*args):
        global callback_called
        callback_called = 'playlist_state_changed', args

    def playlist_update_in_progress(*args):
        global callback_called
        callback_called = 'playlist_update_in_progress', args

    def playlist_metadata_updated(*args):
        global callback_called
        callback_called = 'playlist_metadata_updated', args

    def track_created_changed(*args):
        global callback_called
        callback_called = 'track_created_changed', args

    def track_message_changed(*args):
        global callback_called
        callback_called = 'track_message_changed', args

    def track_seen_changed(*args):
        global callback_called
        callback_called = 'track_seen_changed', args

    def description_changed(*args):
        global callback_called
        callback_called = 'description_changed', args

    def subscribers_changed(*args):
        global callback_called
        callback_called = 'subscribers_changed', args

    def image_changed(*args):
        global callback_called
        callback_called = 'image_changed', args

class TestPlaylistManager(unittest.TestCase):

    artist = mock_artist('artist')
    album = mock_album('album', artist)
    owner = mock_user('owner')
    tracks = [
        (mock_track('track1', [artist], album), owner, 1320961109),
        (mock_track('track2', [artist], album), owner, 1320961109),
        (mock_track('track3', [artist], album), owner, 1320961109),
    ]
    pure_tracks = [t[0] for t in tracks]
    description = 'description'
    num_subscribers = 42
    subscribers = ['sub1', 'sub2', 'sub3']
    image = '01234567890123456789'
    playlist = mock_playlist('foo_', [], owner, subscribers, num_subscribers,
                             description, image)
    def setUp(self):
        global callback_called
        self.manager = MyPlaylistManager()
        callback_called = None

    def test_tracks_added(self):
        global callback_called
        callback = self.manager.tracks_added
        self.playlist.add_tracks_added_callback(callback, self.manager)
        mock_event_trigger(20, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name,"tracks_added")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), list)
        self.assertEqual(len(args[2]), 3)
        self.assertEqual(map(lambda x: x.name(), args[2]), ['foo', 'bar', 'baz'])
        self.assertEqual(type(args[3]), int)

    def test_tracks_moved(self):
        global callback_called
        callback = self.manager.tracks_moved
        self.playlist.add_tracks_moved_callback(callback, self.manager)
        mock_event_trigger(21, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "tracks_moved")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), list)
        self.assertEqual(len(args[2]), 3)
        self.assertEqual(args[2], [0, 1, 2])
        self.assertEqual(type(args[3]), int)

    def test_tracks_removed(self):
        global callback_called
        callback = self.manager.tracks_removed
        self.playlist.add_tracks_removed_callback(callback, self.manager)
        mock_event_trigger(22, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "tracks_removed")
        self.assertEqual(len(args), 4)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), list)
        self.assertEqual(len(args[2]), 3)
        self.assertEqual(args[2], [0, 1, 2])

    def test_playlist_renamed(self):
        global callback_called
        callback = self.manager.playlist_renamed
        self.playlist.add_playlist_renamed_callback(callback, self.manager)
        mock_event_trigger(23, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_renamed")
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())

    def test_playlist_state_changed(self):
        global callback_called
        callback = self.manager.playlist_state_changed
        self.playlist.add_playlist_state_changed_callback(callback, self.manager)
        mock_event_trigger(24, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_state_changed")
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())

    def test_playlist_update_in_progress(self):
        global callback_called
        callback = self.manager.playlist_update_in_progress
        self.playlist.add_playlist_update_in_progress_callback(callback, self.manager)
        mock_event_trigger(25, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_update_in_progress")
        self.assertEqual(len(args), 4)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), bool)
        self.assertEqual(args[2], True)

    def test_playlist_metadata_updated(self):
        global callback_called
        callback = self.manager.playlist_metadata_updated
        self.playlist.add_playlist_metadata_updated_callback(callback, self.manager)
        mock_event_trigger(26, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_metadata_updated")
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())

    def test_track_created_changed(self):
        global callback_called
        callback = self.manager.track_created_changed
        self.playlist.add_track_created_changed_callback(callback, self.manager)
        mock_event_trigger(27, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "track_created_changed")
        self.assertEqual(len(args), 6)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), int)
        self.assertEqual(args[2], 1)
        self.assertEqual(type(args[3]), User)
        self.assertEqual(args[3].canonical_name(), u'foo')
        self.assertEqual(type(args[4]), int)
        self.assertEqual(args[4], 123)

    def test_track_message_changed(self):
        global callback_called
        callback = self.manager.track_message_changed
        self.playlist.add_track_message_changed_callback(callback, self.manager)
        mock_event_trigger(28, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "track_message_changed")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), int)
        self.assertEqual(args[2], 1)
        self.assertEqual(type(args[3]), unicode)
        self.assertEqual(args[3], u'foo')

    def test_track_seen_changed(self):
        global callback_called
        callback = self.manager.track_seen_changed
        self.playlist.add_track_seen_changed_callback(callback, self.manager)
        mock_event_trigger(29, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "track_seen_changed")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), int)
        self.assertEqual(args[2], 1)
        self.assertEqual(type(args[3]), bool)
        self.assertEqual(args[3], False)

    def test_description_changed(self):
        global callback_called
        callback = self.manager.description_changed
        self.playlist.add_description_changed_callback(callback, self.manager)
        mock_event_trigger(30, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "description_changed")
        self.assertEqual(len(args), 4)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), unicode)
        self.assertEqual(args[2], u'foo')

    def test_subscribers_changed(self):
        global callback_called
        callback = self.manager.subscribers_changed
        self.playlist.add_subscribers_changed_callback(callback, self.manager)
        mock_event_trigger(31, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "subscribers_changed")
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())

    def test_image_changed(self):
        global callback_called
        callback = self.manager.image_changed
        self.playlist.add_image_changed_callback(callback, self.manager)
        mock_event_trigger(32, self.playlist)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "image_changed")
        self.assertEqual(len(args), 4)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), Playlist)
        self.assertEqual(args[1].name(), self.playlist.name())
        self.assertEqual(type(args[2]), bytes)
        self.assertEqual(args[2], '01234567890123456789')



__version__ = '1.8.1'

class SpotifyError(Exception):
    pass

# pylint: disable = W0404
def _add_null_handler_for_logging():
    import logging
    try:
        NullHandler = logging.NullHandler  # Python 2.7 and upwards
    except AttributeError:
        class NullHandler(logging.Handler):
            def emit(self, record):
                pass
    logging.getLogger('spotify').addHandler(NullHandler())

_add_null_handler_for_logging()
# pylint: enable = W0404

from spotify._spotify import Session
from spotify._spotify import Track
from spotify._spotify import Artist
from spotify._spotify import Album
from spotify._spotify import Link
from spotify._spotify import Results
from spotify._spotify import Playlist
from spotify._spotify import PlaylistContainer
from spotify._spotify import AlbumBrowser
from spotify._spotify import ArtistBrowser
from spotify._spotify import Image
from spotify._spotify import User
from spotify._spotify import ToplistBrowser

from spotify._spotify import api_version
from spotify._spotify import connect





import unittest
from spotify._mockspotify import ToplistBrowser
from spotify._mockspotify import mock_album, mock_artist, mock_track, mock_toplistbrowse
from spotify._mockspotify import registry_add, registry_clean

callback_called = False
callback_userdata = None

def callback(toplist, userdata):
    global callback_called
    global callback_userdata
    callback_called = True
    callback_userdata = userdata

class TestToplist(unittest.TestCase):

    artist1 = mock_artist('artist1')
    album1  = mock_album('album1', artist1)
    tracks1 = [
        mock_track('track11', [artist1], album1),
        mock_track('track12', [artist1], album1),
        mock_track('track13', [artist1], album1),
    ]
    artist2 = mock_artist('artist2')
    album2  = mock_album('album2', artist2)
    tracks2 = [
        mock_track('track21', [artist2], album2),
        mock_track('track22', [artist2], album2),
        mock_track('track23', [artist2], album2),
    ]
    albums = [album1, album2]
    artists = [artist1, artist2]
    tracks = tracks1 + tracks2
    browser_albums  = mock_toplistbrowse(albums, [], []);
    browser_artists = mock_toplistbrowse([], artists, []);
    browser_tracks  = mock_toplistbrowse([], [], tracks);

    def setUp(self):
        registry_add('spotify:toplist:albums:FR', self.browser_albums)
        registry_add('spotify:toplist:artists:FR', self.browser_artists)
        registry_add('spotify:toplist:tracks:FR', self.browser_tracks)

    def tearDown(self):
        registry_clean()

    def test_is_loaded(self):
        browser = ToplistBrowser('tracks', 'FR')
        self.assertTrue(browser.is_loaded())

    def test_sequence_albums(self):
        browser = ToplistBrowser('albums', 'FR')
        self.assertEqual([a.name() for a in browser],
                        ['album1', 'album2'])

    def test_sequence_artists(self):
        browser = ToplistBrowser('artists', 'FR')
        self.assertEqual([a.name() for a in browser],
                        ['artist1', 'artist2'])

    def test_sequence_tracks(self):
        browser = ToplistBrowser('tracks', 'FR')
        self.assertEqual([t.name() for t in browser],
                        ['track11', 'track12', 'track13',
                         'track21', 'track22', 'track23'])

    def test_callback(self):
        global callback_called
        global callback_userdata
        callback_called = False
        browser = ToplistBrowser('tracks', 'FR', callback, self)
        self.assertTrue(callback_called)
        self.assertEqual(callback_userdata, self)



# encoding: utf-8

import unittest
from nose.tools import raises

from spotify._mockspotify import mock_album, mock_artist, mock_track, mock_playlist
from spotify._mockspotify import mock_playlistcontainer, mock_user, Album
from spotify._mockspotify import mock_session, mock_set_current_session
from spotify._mockspotify import mock_playlistfolder

class TestPlaylistContainer(unittest.TestCase):

    owner = mock_user('owner')
    p1 = mock_playlist('foo', [], owner)
    p2 = mock_playlist('bar', [], owner)

    def test_len(self):
        pc = mock_playlistcontainer(self.owner, [self.p1, self.p2])
        self.assertEqual(len(pc), 2)

    def test_sq_item(self):
        pc = mock_playlistcontainer(self.owner, [self.p1, self.p2])
        self.assertEqual(pc[0].name(), "foo")
        self.assertEqual(pc[1].name(), "bar")

    def test_sq_item_exception(self):
        pc = mock_playlistcontainer(self.owner, [self.p1, self.p2])
        def _():
            return pc[2]
        self.assertRaises(IndexError, _)

    def test_add_new_playlist(self):
        pc = mock_playlistcontainer(self.owner, [])
        pc.add_new_playlist('foo');
        self.assertRaises(ValueError, pc.add_new_playlist, 'foo' * 100)

    def test_playlistfolder(self):
        p1 = mock_playlistfolder("folder_start", "foo", folder_id=42)
        p2 = mock_playlistfolder("folder_end", "")
        pc = mock_playlistcontainer(self.owner, [p1, p2])
        f1,f2  = pc[0],pc[1]
        self.assertEqual(f1.name(), u'foo')
        self.assertEqual(f1.type(), 'folder_start')
        self.assertEqual(f2.type(), 'folder_end')
        self.assertEqual(f1.id(), 42)
        self.assertTrue(f1.is_loaded())

class TestPlaylist(unittest.TestCase):

    artist = mock_artist('artist')
    album = mock_album('album', artist)
    owner = mock_user('owner')
    tracks = [
        (mock_track('track1', [artist], album), owner, 1320961109),
        (mock_track('track2', [artist], album), owner, 1320961109),
        (mock_track('track3', [artist], album), owner, 1320961109),
    ]
    pure_tracks = [t[0] for t in tracks]
    session = mock_session()

    def setUp(self):
        mock_set_current_session(self.session)

    def tearDown(self):
        mock_set_current_session(None)

    def test_name(self):
        playlist = mock_playlist('playlist', [], self.owner)
        self.assertEqual(playlist.name(), 'playlist')

    def test_name_unicode(self):
        playlist = mock_playlist(u'plåylïst', [], self.owner)
        self.assertEqual(playlist.name(), u'plåylïst')

    def test_rename(self):
        playlist = mock_playlist(u'foo', [], self.owner)
        playlist.rename(u'bar')
        self.assertEqual(playlist.name(), u'bar')

    def test_rename_unicode(self):
        playlist = mock_playlist(u'foo', [], self.owner)
        playlist.rename(u'bąr')
        self.assertEqual(playlist.name(), u'bąr')

    @raises(ValueError)
    def test_rename_too_long(self):
        playlist = mock_playlist(u'foo', [], self.owner)
        playlist.rename(u'bar' * 100)

    def test_len(self):
        playlist = mock_playlist(u'foo', self.tracks, self.owner)
        self.assertEqual(len(playlist), 3)

    def test_sq_item(self):
        playlist = mock_playlist(u'foo', self.tracks, self.owner)
        self.assertEqual(playlist[0].name(), 'track1')
        self.assertEqual(playlist[1].name(), 'track2')
        self.assertEqual(playlist[2].name(), 'track3')

    def test_num_subscribers(self):
        playlist = mock_playlist('foo', [], self.owner, num_subscribers=42)
        self.assertEqual(playlist.num_subscribers(), 42)

    def test_subscribers(self):
        playlist = mock_playlist(u'foo', [], self.owner,
                                 subscribers=['foo', 'bar', 'baz'])
        self.assertEqual(playlist.subscribers(), [u'foo', u'bar', u'baz'])

    def test_add_tracks_ok(self):
        playlist = mock_playlist(u'foo', [], self.owner)
        playlist.add_tracks(0, self.pure_tracks)

    @raises(IndexError)
    def test_add_tracks_wrong_position(self):
        playlist = mock_playlist('foo', [], self.owner)
        playlist.add_tracks(99, self.pure_tracks)

    def test_add_tracks_wrong_types(self):
        playlist = mock_playlist('foo', [], self.owner)
        self.assertRaises(TypeError, playlist.add_tracks, 0, True)
        self.assertRaises(TypeError, playlist.add_tracks, [False])

    def test_track_create_time(self):
        playlist = mock_playlist('foo', self.tracks, self.owner)
        self.assertEqual(playlist.track_create_time(0), 1320961109)

    def test_owner(self):
        playlist = mock_playlist('foo', [], self.owner)
        self.assertEqual(playlist.owner().canonical_name(),
                         self.owner.canonical_name())



import unittest
from spotify._mockspotify import ArtistBrowser
from spotify._mockspotify import mock_artistbrowse, mock_artist, mock_album, mock_track
from spotify._mockspotify import registry_add, registry_clean
from tests import SkipTest

callback_called = False
callback_userdata = None

class TestArtistbrowser(unittest.TestCase):

    artist = mock_artist("foo")
    similar_artists = [
        mock_artist('artist1'),
        mock_artist('artist2'),
        mock_artist('artist3'),
    ]
    albums = [
        mock_album("album1", artist),
        mock_album("album2", artist),
        mock_album("album3", artist),
    ]
    tracks = [
        mock_track('track1', [artist], albums[0]),
        mock_track('track2', [artist], albums[0]),
        mock_track('track3', [artist], albums[0]),
    ]
    browser = mock_artistbrowse(artist, tracks, albums, similar_artists, 0)

    def callback(self, browser, userdata):
        global callback_called
        global callback_userdata
        callback_called = True
        callback_userdata = userdata

    def setUp(self):
        registry_add('spotify:artist:foo', self.artist)
        registry_add('spotify:artistbrowse:foo', self.browser)

    def tearDown(self):
        registry_clean()

    def test_is_loaded(self):
        self.assertTrue(self.browser.is_loaded())

    def test_sequence(self):
        self.assertEqual([a.name() for a in self.browser],
                        ['track1', 'track2', 'track3'])

    def test_albums(self):
        self.assertEqual([a.name() for a in self.browser.albums()],
                        ['album1', 'album2', 'album3'])

    def test_artists(self):
        self.assertEqual([a.name() for a in self.browser.similar_artists()],
                        ['artist1', 'artist2', 'artist3'])

    def test_tracks(self):
        self.assertEqual([a.name() for a in self.browser.tracks()],
                        ['track1', 'track2', 'track3'])

    def test_browser(self):
        browser = ArtistBrowser(self.artist)

    def test_browser_with_callback(self):
        global callback_called
        global callback_userdata
        callback_called = False

        browser = ArtistBrowser(self.artist, 'full', self.callback, self)
        self.assertTrue(callback_called)
        self.assertEqual(callback_userdata, self)



"""
The :mod:`spotify.audiosink` module provides audio sink wrappers for different
audio sinks like ALSA, OSS, and PortAudio.
"""

import traceback


AUDIO_SINKS = (
    ('spotify.audiosink.alsa', 'AlsaSink'),
    ('spotify.audiosink.oss', 'OssSink'),
    ('spotify.audiosink.portaudio', 'PortAudioSink'),
    ('spotify.audiosink.gstreamer', 'GstreamerSink'),
)


def import_audio_sink(audio_sinks=None):
    """
    Try to import each audio sink until one is successfully imported.

    The `audio_sinks` parameter specificies what audio sinks to import, in the
    given order. If `audio_sinks` is not provided, the list
    :attr:`spotify.audiosink.AUDIO_SINKS` will be used.

    :param audio_sinks: audio sinks to try to import
    :type audio_sinks: list of two-tuples of (modulename, classname)
    :return: the first audio sink that was successfully imported
    :rtype: class
    :raise: :exc:`ImportError` if no audio sinks can be imported
    """
    if audio_sinks is None:
        audio_sinks = AUDIO_SINKS
    error_messages = []
    for module, cls in audio_sinks:
        try:
            module = __import__(module, fromlist=[cls])
            cls = getattr(module, cls)
            return cls
        except ImportError:
            error_messages.append(
                "Tried to use %s.%s as audio sink, but failed:"
                % (module, cls))
            error_messages.append(traceback.format_exc())
    error_messages.append("Was not able to import any of the audio sinks")
    raise ImportError, "\n".join(error_messages)


class BaseAudioSink(object):
    """
    :class:`BaseAudioSink` provides the interface which is implemented by all
    audio sink wrappers in the :mod:`spotify.audiosink` module.

    The interface is a perfect match for the
    :meth:`spotify.manager.SpotifySessionManager.music_delivery` method, making
    it easy to play audio data received from Spotify.
    """

    def __init__(self, backend=None):
        self._call_cache = {}
        self.backend = backend

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        """
        To use one of the bundled audio controllers in a Spotify client you
        develop, just call this method every time you get audio data from
        Spotify, e.g. from your implementation of
        :meth:`spotify.manager.SpotifySessionManager.music_delivery`.

        :param session: the current session
        :type session: :class:`spotify.Session`
        :param frames: the audio data
        :type frames: :class:`buffer`
        :param frame_size: bytes per frame
        :type frame_size: :class:`int`
        :param num_frames: number of frames in this delivery
        :type num_frames: :class:`int`
        :param sample_type: currently this is always 0 which means 16-bit
            signed native endian integer samples
        :type sample_type: :class:`int`
        :param sample_rate: audio sample rate, in samples per second
        :type sample_rate: :class:`int`
        :param channels: number of audio channels. Currently 1 or 2
        :type channels: :class:`int`
        :return: number of frames consumed
        :rtype: :class:`int`
        """
        raise NotImplementedError

    def end_of_track(self):
        self.backend.next()

    def start(self):
        """
        Should be called when audio output starts.

        This is a hook for the audio sink to do work just before the audio
        starts.
        """
        pass

    def stop(self):
        """
        Should be called when audio output stops.

        This is a hook for the audio sink to do work just after the audio
        stops.
        """
        pass

    def pause(self):
        """
        Should be called when audio output is paused.

        This is a hook for the audio sink to do work when the audio is paused.
        """
        pass

    def _call_if_needed(self, func, *args, **kwargs):
        """
        Calls the given function with the given arguments if the arguments have
        changed since the previous call to the function.
        """
        if (func not in self._call_cache
                or self._call_cache[func] != (args, kwargs)):
            self._call_cache[func] = (args, kwargs)
            func(*args, **kwargs)



# encoding: utf-8

import unittest
from spotify._mockspotify import mock_album, mock_artist, Album

class TestAlbum(unittest.TestCase):

    album = mock_album(u'æâ€êþÿ', mock_artist("bar"), 2006,
                       "01234567890123456789", Album.ALBUM, 1, 1)

    def test_is_loaded(self):
        self.assertEqual(self.album.is_loaded(), 1)

    def test_name(self):
        self.assertEqual(self.album.name(), u'æâ€êþÿ')

    def test_artist(self):
        self.assertEqual(self.album.artist().name(), "bar")

    def test_year(self):
        self.assertEqual(self.album.year(), 2006)

    def test_cover(self):
        self.assertEqual(self.album.cover(), "01234567890123456789")

    def test_type(self):
        self.assertEqual(self.album.type(), Album.ALBUM)




#!/usr/bin/env python

import cmd
import logging
import os
import sys
import threading
import time

from spotify import ArtistBrowser, Link, ToplistBrowser
from spotify.audiosink import import_audio_sink
from spotify.manager import (SpotifySessionManager, SpotifyPlaylistManager,
    SpotifyContainerManager)

AudioSink = import_audio_sink()
container_loaded = threading.Event()

class JukeboxUI(cmd.Cmd, threading.Thread):

    prompt = "jukebox> "

    def __init__(self, jukebox):
        cmd.Cmd.__init__(self)
        threading.Thread.__init__(self)
        self.jukebox = jukebox
        self.playlist = None
        self.track = None
        self.results = False

    def run(self):
        container_loaded.wait()
        container_loaded.clear()
        try:
            self.cmdloop()
        finally:
            self.do_quit(None)

    def do_logout(self, line):
        self.jukebox.session.logout()

    def do_quit(self, line):
        self.jukebox.stop()
        self.jukebox.disconnect()
        print "Goodbye!"
        return True

    def do_list(self, line):
        """ List the playlists, or the contents of a playlist """
        if not line:
            for i, p in enumerate(self.jukebox.ctr):
                if p.is_loaded():
                    print "%3d %s" % (i, p.name())
                else:
                    print "%3d %s" % (i, "loading...")
            print "%3d Starred tracks" % (i + 1,)

        else:
            try:
                p = int(line)
            except ValueError:
                print "that's not a number!"
                return
            if p < 0 or p > len(self.jukebox.ctr):
                print "That's out of range!"
                return
            print "Listing playlist #%d" % p
            if p < len(self.jukebox.ctr):
                playlist = self.jukebox.ctr[p]
            else:
                playlist = self.jukebox.starred
            for i, t in enumerate(playlist):
                if t.is_loaded():
                    print "%3d %s - %s [%s]" % (
                        i, t.artists()[0].name(), t.name(),
                        self.pretty_duration(t.duration()))
                else:
                    print "%3d %s" % (i, "loading...")

    def pretty_duration(self, milliseconds):
        seconds = milliseconds // 1000
        minutes = seconds // 60
        seconds = seconds % 60
        duration = '%02d:%02d' % (minutes, seconds)
        return duration

    def do_play(self, line):
        if not line:
            self.jukebox.play()
            return
        if line.startswith("spotify:"):
            # spotify url
            l = Link.from_string(line)
            if not l.type() == Link.LINK_TRACK:
                print "You can only play tracks!"
                return
            self.jukebox.load_track(l.as_track())
        else:
            try:
                playlist, track = map(int, line.split(' ', 1))
                self.jukebox.load(playlist, track)
            except ValueError:
                try:
                    playlist = int(line)
                    self.jukebox.load_playlist(playlist)
                except ValueError:
                    print("Usage: play [track_link] | "
                          "[playlist] [track] | [playlist]")
                    return
        self.jukebox.play()

    def do_browse(self, line):
        if not line or not line.startswith("spotify:"):
            print "Invalid id provided"
            return
        l = Link.from_string(line)
        if not l.type() in [Link.LINK_ALBUM, Link.LINK_ARTIST]:
            print "You can only browse albums and artists"
            return
        def browse_finished(browser, userdata):
            print "Browse finished, %s" % (userdata)
        self.jukebox.browse(l, browse_finished)

    def print_search_results(self):
        print "Artists:"
        for a in self.results.artists():
            print "    ", Link.from_artist(a), a.name()
        print "Albums:"
        for a in self.results.albums():
            print "    ", Link.from_album(a), a.name()
        print "Tracks:"
        for a in self.results.tracks():
            print "    ", Link.from_track(a, 0), a.name()
        print self.results.total_tracks() - len(self.results.tracks()), \
            "Tracks not shown"

    def do_search(self, line):
        if not line:
            if self.results is False:
                print "No search is in progress"
            elif self.results is None:
                print "Searching is in progress"
            else:
                print "Artists:"
                for a in self.results.artists():
                    print "    ", Link.from_artist(a), a.name()
                print "Albums:"
                for a in self.results.albums():
                    print "    ", Link.from_album(a), a.name()
                print "Tracks:"
                for a in self.results.tracks():
                    print "    ", Link.from_track(a, 0), a.name()
                print self.results.total_tracks() - \
                        len(self.results.tracks()), "Tracks not shown"
        else:
            line = line.decode('utf-8')
            self.results = None
            def search_finished(results, userdata):
                print "\nSearch results received"
                self.results = results
                self.print_search_results()
            self.jukebox.search(line, search_finished)

    def do_queue(self, line):
        if not line:
            for playlist, track in self.jukebox._queue:
                print playlist, track
            return
        try:
            playlist, track = map(int, line.split(' ', 1))
        except ValueError:
            print "Usage: play playlist track"
            return
        self.jukebox.queue(playlist, track)

    def do_stop(self, line):
        self.jukebox.stop()

    def do_next(self, line):
        self.jukebox.next()

    def emptyline(self):
        pass

    def do_watch(self, line):
        if not line:
            print """Usage: watch [playlist]
You will be notified when tracks are added, moved or removed from the
playlist."""
        else:
            try:
                p = int(line)
            except ValueError:
                print "That's not a number!"
                return
            if p < 0 or p >= len(self.jukebox.ctr):
                print "That's out of range!"
                return
            self.jukebox.watch(self.jukebox.ctr[p])

    def do_unwatch(self, line):
        if not line:
            print "Usage: unwatch [playlist]"
        else:
            try:
                p = int(line)
            except ValueError:
                print "That's not a number!"
                return
            if p < 0 or p >= len(self.jukebox.ctr):
                print "That's out of range!"
                return
            self.jukebox.watch(self.jukebox.ctr[p], True)

    def do_toplist(self, line):
        usage = "Usage: toplist (albums|artists|tracks) (GB|FR|..|all|current)"
        if not line:
            print usage
        else:
            args = line.split(' ')
            if len(args) != 2:
                print usage
            else:
                self.jukebox.toplist(*args)

    def do_shell(self, line):
        self.jukebox.shell()

    def do_add_new_playlist(self, line):
        if not line:
            print "Usage: add_new_playlist <name>"
        else:
            new_playlist = self.jukebox.ctr.add_new_playlist(
                line.decode('utf-8'))

    def do_add_to_playlist(self, line):
        usage = "Usage: add_to_playlist <playlist_index> <insert_point>" + \
                " <search_result_indecies>"
        if not line:
            print usage
            return
        args = line.split(' ')
        if len(args) < 3:
            print usage
        else:
            if not self.results:
                print "No search results"
            else:
                index = int(args.pop(0))
                insert = int(args.pop(0))
                artists = self.results.artists()
                tracks = self.results.tracks()
                for i in args:
                    for a in tracks[int(i)].artists():
                        print u'{0}. {1} - {2} '.format(
                            i, a.name(), tracks[int(i)].name())
                print u'adding them to {0} '.format(
                    self.jukebox.ctr[index].name())
                self.jukebox.ctr[index].add_tracks(
                    insert, [tracks[int(i)] for i in args])

    do_ls = do_list
    do_EOF = do_quit


## playlist callbacks ##
class JukeboxPlaylistManager(SpotifyPlaylistManager):
    def tracks_added(self, p, t, i, u):
        print 'Tracks added to playlist %s' % p.name()

    def tracks_moved(self, p, t, i, u):
        print 'Tracks moved in playlist %s' % p.name()

    def tracks_removed(self, p, t, u):
        print 'Tracks removed from playlist %s' % p.name()

## container calllbacks ##
class JukeboxContainerManager(SpotifyContainerManager):
    def container_loaded(self, c, u):
        container_loaded.set()

    def playlist_added(self, c, p, i, u):
        print 'Container: playlist "%s" added.' % p.name()

    def playlist_moved(self, c, p, oi, ni, u):
        print 'Container: playlist "%s" moved.' % p.name()

    def playlist_removed(self, c, p, i, u):
        print 'Container: playlist "%s" removed.' % p.name()

class Jukebox(SpotifySessionManager):

    queued = False
    playlist = 2
    track = 0
    appkey_file = os.path.join(os.path.dirname(__file__), 'spotify_appkey.key')

    def __init__(self, *a, **kw):
        SpotifySessionManager.__init__(self, *a, **kw)
        self.audio = AudioSink(backend=self)
        self.ui = JukeboxUI(self)
        self.ctr = None
        self.playing = False
        self._queue = []
        self.playlist_manager = JukeboxPlaylistManager()
        self.container_manager = JukeboxContainerManager()
        self.track_playing = None
        print "Logging in, please wait..."

    def new_track_playing(self, track):
        self.track_playing = track

    def logged_in(self, session, error):
        if error:
            print error
            return
        self.session = session
        self.ctr = session.playlist_container()
        self.container_manager.watch(self.ctr)
        self.starred = session.starred()
        self.ui.start()

    def logged_out(self, session):
        self.ui.cmdqueue.append("quit")

    def load_track(self, track):
        if self.playing:
            self.stop()
        self.new_track_playing(track)
        self.session.load(track)
        print "Loading %s" % track.name()

    def load(self, playlist, track):
        if self.playing:
            self.stop()
        if 0 <= playlist < len(self.ctr):
            pl = self.ctr[playlist]
        elif playlist == len(self.ctr):
            pl = self.starred
        spot_track = pl[track]
        self.new_track_playing(spot_track)
        self.session.load(spot_track)
        print "Loading %s from %s" % (spot_track.name(), pl.name())

    def load_playlist(self, playlist):
        if self.playing:
            self.stop()
        if 0 <= playlist < len(self.ctr):
            pl = self.ctr[playlist]
        elif playlist == len(self.ctr):
            pl = self.starred
        print "Loading playlist %s" % pl.name()
        if len(pl):
            print "Loading %s from %s" % (pl[0].name(), pl.name())
            self.new_track_playing(pl[0])
            self.session.load(pl[0])
        for i, track in enumerate(pl):
            if i == 0:
                continue
            self._queue.append((playlist, i))

    def queue(self, playlist, track):
        if self.playing:
            self._queue.append((playlist, track))
        else:
            print 'Loading %s', track.name()
            self.load(playlist, track)
            self.play()

    def play(self):
        self.audio.start()
        self.session.play(1)
        print "Playing"
        self.playing = True

    def stop(self):
        self.session.play(0)
        print "Stopping"
        self.playing = False
        self.audio.stop()

    def music_delivery_safe(self, *args, **kwargs):
        return self.audio.music_delivery(*args, **kwargs)

    def next(self):
        self.stop()
        if self._queue:
            t = self._queue.pop(0)
            self.load(*t)
            self.play()
        else:
            self.stop()

    def end_of_track(self, sess):
        self.audio.end_of_track()

    def search(self, *args, **kwargs):
        self.session.search(*args, **kwargs)

    def browse(self, link, callback):
        if link.type() == link.LINK_ALBUM:
            browser = self.session.browse_album(link.as_album(), callback)
            while not browser.is_loaded():
                time.sleep(0.1)
            for track in browser:
                print track.name()
        if link.type() == link.LINK_ARTIST:
            browser = ArtistBrowser(link.as_artist())
            while not browser.is_loaded():
                time.sleep(0.1)
            for album in browser:
                print album.name()

    def watch(self, p, unwatch=False):
        if not unwatch:
            print "Watching playlist: %s" % p.name()
            self.playlist_manager.watch(p)
        else:
            print "Unatching playlist: %s" % p.name()
            self.playlist_manager.unwatch(p)

    def toplist(self, tl_type, tl_region):
        print repr(tl_type)
        print repr(tl_region)
        def callback(tb, ud):
            for i in xrange(len(tb)):
                print '%3d: %s' % (i+1, tb[i].name())

        tb = ToplistBrowser(tl_type, tl_region, callback)

    def shell(self):
        import code
        shell = code.InteractiveConsole(globals())
        shell.interact()

if __name__ == '__main__':
    import optparse
    op = optparse.OptionParser(version="%prog 0.1")
    op.add_option("-u", "--username", help="Spotify username")
    op.add_option("-p", "--password", help="Spotify password")
    op.add_option("-v", "--verbose", help="Show debug information",
        dest="verbose", action="store_true")
    (options, args) = op.parse_args()
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)
    session_m = Jukebox(options.username, options.password, True)
    session_m.connect()



import unittest
from spotify._mockspotify import mock_playlist, mock_event_trigger, mock_user
from spotify._mockspotify import mock_playlistcontainer
from spotify.manager import SpotifyContainerManager
import spotify

callback_called = None

class MyContainerManager(SpotifyContainerManager):

    def container_loaded(*args):
        global callback_called
        callback_called = 'container_loaded', args

    def playlist_added(*args):
        global callback_called
        callback_called = 'playlist_added', args

    def playlist_moved(*args):
        global callback_called
        callback_called = 'playlist_moved', args

    def playlist_removed(*args):
        global callback_called
        callback_called = 'playlist_removed', args


class TestPlaylistManager(unittest.TestCase):

    def setUp(self):
        self.manager = MyContainerManager()
        self.container = mock_playlistcontainer(mock_user('user'), [])

    def test_container_loaded(self):
        global callback_called
        callback = self.manager.container_loaded
        self.container.add_loaded_callback(callback, self.manager)
        mock_event_trigger(40, self.container)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "container_loaded")
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), spotify._mockspotify.PlaylistContainer)

    def test_playlist_added(self):
        global callback_called
        callback = self.manager.playlist_added
        self.container.add_playlist_added_callback(callback, self.manager)
        mock_event_trigger(41, self.container)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_added")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), spotify._mockspotify.PlaylistContainer)
        self.assertEqual(type(args[2]), spotify._mockspotify.Playlist)
        self.assertEqual(args[2].name(), 'P')
        self.assertEqual(type(args[3]), int)

    def test_playlist_moved(self):
        global callback_called
        callback = self.manager.playlist_moved
        self.container.add_playlist_moved_callback(callback, self.manager)
        mock_event_trigger(42, self.container)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_moved")
        self.assertEqual(len(args), 6)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), spotify._mockspotify.PlaylistContainer)
        self.assertEqual(type(args[2]), spotify._mockspotify.Playlist)
        self.assertEqual(args[2].name(), 'P')
        self.assertEqual(type(args[3]), int)
        self.assertEqual(type(args[4]), int)

    def test_playlist_removed(self):
        global callback_called
        callback = self.manager.playlist_removed
        self.container.add_playlist_removed_callback(callback, self.manager)
        mock_event_trigger(43, self.container)

        self.assertNotEqual(callback_called, None)
        name, args = callback_called
        self.assertEqual(name, "playlist_removed")
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0], self.manager)
        self.assertEqual(type(args[1]), spotify._mockspotify.PlaylistContainer)
        self.assertEqual(type(args[2]), spotify._mockspotify.Playlist)
        self.assertEqual(args[2].name(), 'P')
        self.assertEqual(type(args[3]), int)



import unittest
from spotify._mockspotify import mock_albumbrowse, mock_album, mock_artist, \
                                 mock_track
from spotify._mockspotify import Album, AlbumBrowser
from spotify._mockspotify import registry_add, registry_clean

callback_called = False
callback_userdata = None

class TestAlbumbrowser(unittest.TestCase):

    artist = mock_artist("foo")
    album = mock_album("bar", artist)
    tracks = [
        mock_track("baz1", [artist], album),
        mock_track("baz2", [artist], album),
        mock_track("baz3", [artist], album),
    ]
    browser = mock_albumbrowse(album, tracks, artist=artist, error=0)

    def callback(self, browser, userdata):
        global callback_called
        global callback_userdata
        callback_called = True
        callback_userdata = userdata

    def setUp(self):
        registry_add('spotify:album:1234', self.album)
        registry_add('spotify:albumbrowse:1234', self.browser)

    def tearDown(self):
        registry_clean()

    def test_is_loaded(self):
        assert self.browser.is_loaded()

    def test_sequence(self):
        assert len(self.browser) == 3
        assert self.browser[0].name() == 'baz1'
        assert self.browser[1].name() == 'baz2'
        assert self.browser[2].name() == 'baz3'

    def test_browser(self):
        browser = AlbumBrowser(self.album)

    def test_browser_with_callback(self):
        global callback_called
        global callback_userdata
        callback_called = False

        browser = AlbumBrowser(self.album, self.callback, self)
        self.assertTrue(callback_called)
        self.assertEqual(callback_userdata, self)



#! /usr/bin/env python

from distutils.core import setup, Extension
import os
import re
import sys

def get_version():
    init_py = open('spotify/__init__.py').read()
    metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", init_py))
    return metadata['version']

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

def with_mock():
    """
    Checks wether the user wants to build the mockmodule (off by default).
    """
    try:
        sys.argv.remove('--with-mock')
        return True
    except ValueError:
        return False

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
project_dir = 'spotify'

for dirpath, dirnames, filenames in os.walk(project_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath,
            [os.path.join(dirpath, f) for f in filenames]])

spotify_ext = Extension('spotify._spotify',
    [
        'src/module.c',
        'src/session.c',
        'src/link.c',
        'src/track.c',
        'src/album.c',
        'src/albumbrowser.c',
        'src/artist.c',
        'src/artistbrowser.c',
        'src/search.c',
        'src/playlist.c',
        'src/playlistcontainer.c',
        'src/playlistfolder.c',
        'src/image.c',
        'src/user.c',
        'src/pyspotify.c',
        'src/toplistbrowser.c',
    ],
    include_dirs=['src'],
    libraries=['spotify'],
)

mockspotify_ext = Extension('spotify._mockspotify',
    [
        'src/mockmodule.c',
        'src/session.c',
        'src/link.c',
        'src/track.c',
        'src/album.c',
        'src/albumbrowser.c',
        'src/artist.c',
        'src/artistbrowser.c',
        'src/search.c',
        'src/playlist.c',
        'src/playlistcontainer.c',
        'src/playlistfolder.c',
        'src/image.c',
        'src/user.c',
        'src/pyspotify.c',
        'src/toplistbrowser.c',
    ],
    include_dirs=['src'],
    libraries=['mockspotify'],
)

modules = [spotify_ext]
if with_mock():
    modules.append(mockspotify_ext)

setup(
    name='pyspotify',
    version=get_version(),
    description='Python wrapper for libspotify',
    long_description=open('README.rst').read(),
    author='Doug Winter',
    author_email='doug.winter@isotoma.com',
    url='http://pyspotify.mopidy.com/',
    packages=packages,
    data_files=data_files,
    ext_modules=modules,
)



import unittest
from spotify._mockspotify import mock_artist, mock_album, mock_search, mock_track

class TestSearch(unittest.TestCase):

    artist1 = mock_artist('artist1')
    album1  = mock_album('album1', artist1)
    tracks1 = [
        mock_track('track11', [artist1], album1),
        mock_track('track12', [artist1], album1),
        mock_track('track13', [artist1], album1),
    ]
    artist2 = mock_artist('artist2')
    album2  = mock_album('album2', artist2)
    tracks2 = [
        mock_track('track21', [artist2], album2),
        mock_track('track22', [artist2], album2),
        mock_track('track23', [artist2], album2),
    ]
    albums = [album1, album2]
    artists = [artist1, artist2]
    tracks = tracks1 + tracks2
    search = mock_search('query', tracks, albums, artists, 6, 2, 2, 'query2', 0)

    def test_search_is_loaded(self):
        self.assertEqual(self.search.is_loaded(), True)

    def test_artists(self):
        self.assertEqual([a.name() for a in self.search.artists()],
                        ['artist1', 'artist2'])

    def test_albums(self):
        self.assertEqual([a.name() for a in self.search.albums()],
                        ['album1', 'album2'])

    def test_tracks(self):
        self.assertEqual([t.name() for t in self.search.tracks()],
                        ['track11', 'track12', 'track13',
                         'track21', 'track22', 'track23'])

    def test_query(self):
        self.assertEqual(self.search.query(), "query")

    def test_error(self):
        self.assertEqual(self.search.error(), 0)

    def test_did_you_mean(self):
        self.assertEqual(self.search.did_you_mean(), "query2")

    def test_totals(self):
        self.assertEqual(self.search.total_albums(), 2)
        self.assertEqual(self.search.total_artists(), 2)
        self.assertEqual(self.search.total_tracks(), 6)




import alsaaudio
import sys

from spotify.audiosink import BaseAudioSink

class AlsaSink(BaseAudioSink):
    """Audio sink wrapper for systems with ALSA, e.g. most Linux systems"""

    def __init__(self, **kwargs):
        super(AlsaSink, self).__init__(**kwargs)
        self._mode = kwargs.get('mode', alsaaudio.PCM_NORMAL)
        self._device = None
        if sys.byteorder == 'little':
            self._format = alsaaudio.PCM_FORMAT_S16_LE
        elif sys.byteorder == 'big':
            self._format = alsaaudio.PCM_FORMAT_S16_BE

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        if self._device is None:
            self._device = alsaaudio.PCM(mode=self._mode)
        self._call_if_needed(self._device.setformat, self._format)
        self._call_if_needed(self._device.setperiodsize, num_frames)
        self._call_if_needed(self._device.setrate, sample_rate)
        self._call_if_needed(self._device.setchannels, channels)
        return self._device.write(frames)



import ossaudiodev
import sys

from spotify.audiosink import BaseAudioSink

class OssSink(BaseAudioSink):
    """Audio sink wrapper for systems with OSS, e.g. older Linux systems"""

    def __init__(self, **kwargs):
        super(OssSink, self).__init__(**kwargs)
        self._device = None
        if sys.byteorder == 'little':
            self._format = ossaudiodev.AFMT_S16_LE
        elif sys.byteorder == 'big':
            self._format = ossaudiodev.AFMT_S16_BE

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        if num_frames == 0:
            if self._device is not None:
                self._device.close()
                self._device = None
            return 0
        if self._device is None:
            self._device = ossaudiodev.open('w')
            self._device.setparameters(self._format, channels, sample_rate)
        self._device.write(frames)
        return num_frames
